<?php

namespace Drupal\pki_ra\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * PKI Registration Authority settings form for ID quality.
 */
class PKIRAIdQualitySettingsForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'pki_ra_settings_id_quality_form';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['pki_ra.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('pki_ra.settings');

    $form['service_url'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Service URL'),
      '#description' => $this->t('The URL of the Identity Quality Assurance (IDQA) service endpoint for scores such as <em>https://idqa.example.com/score</em>. Non-HTTPS URLs should only be used in dev/testing environments. Leave blank if not using one.'),
      '#default_value' => $config->get('idqa.service_url') ?: '',
    ];

    $form['enrolment_score'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Enrolment score'),
      '#description' => $this->t('Initial IDQA score assigned to the user upon enrolment, reported to the service. Must be of the form <em>12345678</em>, 8 digits with each being 0-9 (inclusive).'),
      '#default_value' => $config->get('idqa.enrolment_score') ?: '',
    ];

    $form['signing_credential'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Signing credential'),
      '#description' => $this->t('Used for RA authentication, and provided to you in advance. Necessary for signing the request sent to the service.'),
      '#default_value' => $config->get('idqa.signing_credential') ?: '',
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->config('pki_ra.settings')
      ->set('idqa.service_url', $form_state->getValue('service_url'))
      ->set('idqa.enrolment_score', $form_state->getValue('enrolment_score'))
      ->set('idqa.signing_credential', $form_state->getValue('signing_credential'))
      ->save();

    parent::submitForm($form, $form_state);
  }

}
