<?php

namespace Drupal\pki_ra\Services;

use Drupal\pki_ra\Misc\IdqaScore;

class PkiIdqaScoreReportingService extends PkiService {

  protected $score;

  public function __construct(IdqaScore $score) {
    $this->url = $this->getServiceUrl();
    $this->score = $score;
  }

  public static function isEnabled() {
    return !empty(self::getServiceUrl());
  }

  protected static function getServiceUrl() {
    return \Drupal::config('pki_ra.settings')->get('idqa.service_url');
  }

  public function report() {
    try {
      $request = \Drupal::httpClient()->post($this->url, [
        'json' => $this->getDataToSend(),
        'headers' => $this->getHeadersToSend(),
      ]);
    }
    catch (Exception $e) {
      watchdog_exception('pki_ra', $e->getMessage());
    }

    $this->status = $request->getStatusCode();
    $this->headers = $request->getHeaders();
    $this->response = json_decode((string) $request->getBody(), TRUE);
    return $this;
  }

  protected function sign($data) {
    $credential = \Drupal::config('pki_ra.settings')->get('idqa.signing_credential');
    $serialized_data = serialize($data);

    if (!openssl_sign($serialized_data, $signature, $credential, OPENSSL_ALGO_SHA256)) {
      throw new \Exception('Could not sign data to be sent to the IDQA service: ' . $serialized_data);
    }
    return $signature;
  }

  protected function getDataToSend() {
    $data = [
      'certificate' => $this->score->getX509Certificate(),
      'score' => $this->score->calculate(),
    ];
    $data['signature'] = base64_encode($this->sign($data));
    return $data;
  }

  protected function getHeadersToSend() {
    return [];
  }

  public function getLogData($certificate_node_id) {
    return [
      '%score' => $this->score->calculate(),
      '%id' => $certificate_node_id,
      '%url' => $this->url,
      '%code' => $this->getResponseStatus(),
      '%message' => isset($this->getResponseData()['error']) ?: '',
    ];
  }
}
